Package: InteractionSet
Version: 1.41.0
Date: 2025-07-15
Title: Base Classes for Storing Genomic Interaction Data
Authors@R: c(person("Aaron", "Lun", role = c("aut", "cre"), email = "infinite.monkeys.with.keyboards@gmail.com"),
        person("Malcolm", "Perry", role = "aut", email = "mgperry32@gmail.com"),
        person("Elizabeth", "Ing-Simmons", role = "aut"))
Depends: GenomicRanges, SummarizedExperiment
Imports: methods, Matrix, Rcpp, BiocGenerics, S4Vectors (>= 0.27.12),
        IRanges, Seqinfo
Suggests: testthat, knitr, rmarkdown, BiocStyle
LinkingTo: Rcpp
biocViews: Infrastructure, DataRepresentation, Software, HiC
Description: Provides the GInteractions, InteractionSet and ContactMatrix
        objects and associated methods for storing and manipulating genomic
        interaction data from Hi-C and ChIA-PET experiments.
License: GPL-3
NeedsCompilation: yes
SystemRequirements: C++11
VignetteBuilder: knitr
Collate: AllGenerics.R AllClasses.R ContactMatrix-methods.R
        GInteractions-methods.R InteractionSet-methods.R
        GRanges-methods.R getset.R swapAnchors.R pairs.R conversion.R
        linearize.R boundingBox.R distances.R overlaps.R linkOverlaps.R
        matching.R updateObject.R
RoxygenNote: 7.1.1
git_url: https://git.bioconductor.org/packages/InteractionSet
git_branch: devel
git_last_commit: 275c24c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 23:19:23 UTC; biocbuild
Author: Aaron Lun [aut, cre],
  Malcolm Perry [aut],
  Elizabeth Ing-Simmons [aut]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
