useDynLib(InteractionSet, .registration=TRUE, .fixes="cxx_")

# Exporting the InteractionSet class.
exportClasses(
    GInteractions, StrictGInteractions, ReverseStrictGInteractions,
    InteractionSet, 
    ContactMatrix
)

exportMethods(
    GInteractions,
    anchors, anchorIds, regions, reduceRegions, swapAnchors,
    "regions<-", "replaceRegions<-",  "appendRegions<-", "anchorIds<-",
    seqinfo, "seqinfo<-",
    pairs, 
    findOverlaps, countOverlaps, overlapsAny,
    InteractionSet, 
    interactions,
    "interactions<-",
    pairdist, intrachr, linearize, linkOverlaps, boundingBox,
    ContactMatrix, 
    "as.matrix<-",
    inflate, deflate,
    updateObject
)

export(
    makeGInteractionsFromGRangesPairs
)

# Requires several imports.
import(methods, BiocGenerics, S4Vectors, IRanges,
    Seqinfo, GenomicRanges, SummarizedExperiment)

importFrom("Matrix", "Matrix", "sparseMatrix")

importFrom("Rcpp", sourceCpp)
