# S4 classes for genomic interaction data

|Environment|Status|
|---|---|
|BioC-release|[![Release OK](https://bioconductor.org/shields/build/release/bioc/InteractionSet.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/InteractionSet/)|
|BioC-devel|[![Devel OK](https://bioconductor.org/shields/build/devel/bioc/InteractionSet.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/InteractionSet/)|

The **InteractionSet** package implements a variety of S4 classes for storing and manipulating genomic interaction data from Hi-C and ChIA-PET experiments.
Check out the user's guide on the [Bioconductor landing page](https://bioconductor.org/packages/devel/bioc/html/InteractionSet.html) for more details.
